@@ -72,5 +72,9 @@
 #include "regex.h"
 #include "libi3.h"
 #include "startup.h"
+#include "scratchpad.h"
+#include "commands.h"
+#include "commands_parser.h"
+#include "fake_outputs.h"
 
 #endif
