@@ -2,7 +2,7 @@
  * vim:ts=4:sw=4:expandtab
  *
  * i3 - an improved dynamic tiling window manager
- * © 2009-2011 Michael Stapelberg and contributors (see also: LICENSE)
+ * © 2009-2012 Michael Stapelberg and contributors (see also: LICENSE)
  *
  * click.c: Button press (mouse click) events.
  *
@@ -193,24 +193,37 @@ static int route_click(Con *con, xcb_button_press_event_t *event, const bool mod
          event->detail == XCB_BUTTON_INDEX_5)) {
         DLOG("Scrolling on a window decoration\n");
         orientation_t orientation = (con->parent->layout == L_STACKED ? VERT : HORIZ);
-        if (event->detail == XCB_BUTTON_INDEX_4)
+        /* To prevent scrolling from going outside the container (see ticket
+         * #557), we first check if scrolling is possible at all. */
+        Con *focused = con_descend_focused(con->parent);
+        bool scroll_prev_possible = (TAILQ_PREV(focused, nodes_head, nodes) != NULL);
+        bool scroll_next_possible = (TAILQ_NEXT(focused, nodes) != NULL);
+        if (event->detail == XCB_BUTTON_INDEX_4 && scroll_prev_possible)
             tree_next('p', orientation);
-        else tree_next('n', orientation);
+        else if (event->detail == XCB_BUTTON_INDEX_5 && scroll_next_possible)
+            tree_next('n', orientation);
         goto done;
     }
 
-    /* 2: focus this con */
+    /* 2: focus this con. If the workspace is on another output we need to
+     * do a workspace_show in order for i3bar (and others) to notice the
+     * change in workspace. */
+    Con *ws = con_get_workspace(con);
+    Con *focused_workspace = con_get_workspace(focused);
+
+    if (ws != focused_workspace)
+        workspace_show(ws);
+    focused_id = XCB_NONE;
     con_focus(con);
 
     /* 3: For floating containers, we also want to raise them on click.
      * We will skip handling events on floating cons in fullscreen mode */
-    Con *ws = con_get_workspace(con);
     Con *fs = (ws ? con_get_fullscreen_con(ws, CF_OUTPUT) : NULL);
     if (floatingcon != NULL && fs == NULL) {
         floating_raise_con(floatingcon);
 
         /* 4: floating_modifier plus left mouse button drags */
-        if (mod_pressed && event->detail == 1) {
+        if (mod_pressed && event->detail == XCB_BUTTON_INDEX_1) {
             floating_drag_window(floatingcon, event);
             return 1;
         }
@@ -259,7 +272,9 @@ static int route_click(Con *con, xcb_button_press_event_t *event, const bool mod
             return 1;
     }
     /* 8: otherwise, check for border/decoration clicks and resize */
-    else if (dest == CLICK_BORDER || dest == CLICK_DECORATION) {
+    else if ((dest == CLICK_BORDER || dest == CLICK_DECORATION) &&
+             (event->detail == XCB_BUTTON_INDEX_1 ||
+              event->detail == XCB_BUTTON_INDEX_3)) {
         DLOG("Trying to resize (tiling)\n");
         tiling_resize(con, event, dest);
     }
